<?php

class Ddm_Product_Reviews
{
	private $_db;
	private $_settings;
	
	/**
	 * The constructor
	 *
	 * @param string $db pass database handler
	 * @param string $settings pass global application settings
	 * @return void
	 */
	public function __construct()
	{
		$this->_db = db();
		$this->_settings = settings();
	}
	
	/**
	 * Adds a product review to the database
	 *
	 * @param int $product_id The product being reviewed
	 * @param int $user_id The user id of the reviewer
	 * @param array $review_data Array with review specific keys ('rating','review_title','review_text')
	 * @return int Returns review id on success, FALSE on failure
	 */
	function addReview($review_data)
	{
		$pid = $review_data['pid'];
		$user_id = $review_data['user_id'];
		
		// make sure we have review data
		if (empty($review_data))
		{
			return false;
		}
	
		// check if the user has purchased this product in the past (complete)
		$purchased = intval((bool) $this->isPurchasedProduct($pid, $user_id));
		
		// auto approve the review if the admin has enabled these settings
		$approved =	(
			$this->_settings['ReviewsAutoApprove']  && 
			intval($review_data['rating']) >= intval($this->_settings['ReviewsAutoApproveRating']) && 
			$purchased == intval($this->_settings['ReviewsAutoApprovePurchased'])
		) ? 1 : 0;
		
		// reset the database handler and prepare the insert
		$this->_db->reset();
		$this->_db->assign("pid", intval($pid));
		$this->_db->assign("user_id", intval($user_id));
		$this->_db->assign("date_created", "NOW()");
		$this->_db->assignStr("review_title", htmlentities(strip_tags($review_data['review_title'])));
		$this->_db->assignStr("review_text", htmlentities(strip_tags($review_data['review_text'])));
		$this->_db->assignStr("rating", $review_data['rating']);
		$this->_db->assignStr("approved", $approved);
		$this->_db->assignStr("is_visible", $approved);
		$this->_db->assignStr("purchased", $purchased);
		
		// insert and return review id
		$reviewId = $this->_db->insert(DB_PREFIX."products_reviews");
		
		$rating = $this->getProductRating($pid);
		
		$this->_db->reset();
		$this->_db->assignStr("rating", $rating["rating"]);
		$this->_db->assignStr("reviews_count", $rating["total_reviews"]);
		$this->_db->update(DB_PREFIX."products", "WHERE pid='".intval($pid)."'");
		
		return $reviewId;
	}
	
	
	
	/**
	 * Checks if product has been purchased (order status 'Completed') by the user
	 *
	 * @param int $product_id 
	 * @param int $user_id 
	 * @return bool 
	 */
	function isPurchasedProduct($product_id, $user_id)
	{
		$this->_db->query('
			SELECT COUNT(0) 
			FROM '.DB_PREFIX.'orders_content oc
			JOIN '.DB_PREFIX.'orders o ON o.oid = oc.oid
			WHERE 
				o.status = "Completed"
				AND oc.pid = '.intval($product_id).'
				AND o.uid = '.intval($user_id).'
		');
		
		return (bool)current($this->_db->moveNext());
	}
	
	/**
	 * Gets the reviews for the given product ids
	 *
	 * @param mixed $product_id Product ID or list of IDs in array
	 * @return array Returns an array of reviews
	 */
	function getReviews($product_ids)
	{
		$ids = is_array($product_ids) ? $product_ids : array($product_ids);
		
		$this->_db->query('
			SELECT
				products_reviews.*,
				date_created as date_created1,
				DATE_FORMAT(date_created, "'.$this->_settings["LocalizationDateTimeFormat"].'") AS date_created,
				users.login AS user_login
			FROM '.DB_PREFIX.'products_reviews AS products_reviews
			LEFT JOIN '.DB_PREFIX.'users AS users ON users.uid=products_reviews.user_id AND users.removed="No"
			WHERE 
				pid IN ('.implode(',', $ids).')
				AND is_visible = 1
				AND approved = 1
			ORDER BY date_created1 DESC
		');
		
		return $this->_db->getRecords();
	}
	
	/**
	 * Returns reviews coung by given product id
	 * 
	 * @param $pid
	 * @return int
	 */
	function getReviewsCount($pid)
	{
		$this->_db->query('
			SELECT 
				COUNT(*) AS c
			FROM '.DB_PREFIX.'products_reviews 
			WHERE 
				pid="'.intval($pid).'" 
				AND is_visible = 1
				AND approved = 1
		');
		return $this->_db->movenext() ? $this->_db->col["c"] : 0;	
	}
	
	/**
	 * Gets a product rating along with other useful information like total number of reviews
	 *
	 * @param int $product_id 
	 * @return array Rating details
	 */
	public function getProductRating($product_id)
	{
		$this->_db->query('
			SELECT
				AVG(rating) as rating,
				COUNT(0) as total_reviews
			FROM '.DB_PREFIX.'products_reviews
			WHERE pid = '.intval($product_id).'
			AND approved = 1
		');
		
		return current($this->_db->getRecords());
	}
	
	/**
	 * Checks if the user has written a product review already for the given product id
	 *
	 * @param int $product_id 
	 * @param int $user_id 
	 * @return void
	 * @author Sebastian Nievas
	 */
	public function hasWrittenReview($product_id, $user_id)
	{
		$this->_db->query('
			SELECT COUNT(0) 
			FROM '.DB_PREFIX.'products_reviews pr
			WHERE 
				pr.pid = '.intval($product_id).'
				AND pr.user_id = '.intval($user_id).'
		');
		
		return (bool)current($this->_db->moveNext());
	}
}
